/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "can_example.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/
#define REC_TASK_PRIO   ( tskIDLE_PRIORITY + 2 )
#define SEND_TASK_PRIO    ( tskIDLE_PRIORITY + 3 )
#define REG_TASK_PRIO    ( tskIDLE_PRIORITY + 1 )
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
xQueueHandle xQueueFromCAN2,xQueueToCAN1;

CanRxMsg Message;

GPIO_InitTypeDef GPIO_InitStructure;

/* Private function prototypes -----------------------------------------------*/
void LED_Init(void);
uint8_t CAN_Send(CAN_TypeDef* CANx,CanTxMsg *Message);

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Send received Message
  * @param  pvParameters not used
  * @retval None
  */
void vTask_Send_To_CAN1( void * pvParameters )
{
	CanRxMsg RxMessage;
	uint8_t transmitstatus,queue=0;
	static portTickType xLastWakeTime;

	xLastWakeTime = xTaskGetTickCount();


	while(1)
		{
		queue=xQueueReceive( xQueueToCAN1, &( RxMessage ), 0 );

					while(queue!=0)
					{

						transmitstatus=CAN_Send(CAN1,(CanTxMsg*)(&RxMessage));

						    if(transmitstatus==4)
						    	{
						    		while(CAN_GetFlagStatus(CAN1,CAN_FLAG_RQCP2)==0)
						    			;

						    		transmitstatus=CAN_Send(CAN1,(CanTxMsg*)&RxMessage);
						    	}
						 queue=xQueueReceive( xQueueToCAN1, &( RxMessage ), 0 );

					}

					GPIO_ResetBits(GPIOE, GPIO_Pin_12);

			    	 vTaskDelayUntil( &xLastWakeTime, 5 );

			     	GPIO_SetBits(GPIOE, GPIO_Pin_12);
		}


}

/**
  * @brief  Get received Message
  * @param  pvParameters not used
  * @retval None
  */
void vTask_CAN2_MessageReceived ( void * pvParameters )
{
	uint8_t i,queue=0;
    CanRxMsg RxMessage;
	static portTickType xLastWakeTime;

	xLastWakeTime = xTaskGetTickCount();


    for(;;)
    {
    	queue=xQueueReceive( xQueueFromCAN2, &( RxMessage ), 0 );

    	while(queue!=0)
    		{

    			xQueueSendToBack( xQueueToCAN1, &RxMessage, 0 );

    			queue=xQueueReceive( xQueueFromCAN2, &( RxMessage ), 0 );

    		}

   	 vTaskDelayUntil( &xLastWakeTime, 2 );
    }

}


/**
  * @brief  Fake control function
  * @param  pvParameters not used
  * @retval None
  */
void vTask_Reg ( void * pvParameters )
{
	volatile uint32_t important_var;
	static uint16_t i;
	static portTickType xLastWakeTime;

	xLastWakeTime = xTaskGetTickCount();

    for(;;)
    {

    	for(i=0;i<50000;i++)
    	{
    		important_var++;
    	}

    	GPIO_ResetBits(GPIOE, GPIO_Pin_9);

    	 vTaskDelayUntil( &xLastWakeTime, 10 );

    	 GPIO_SetBits(GPIOE, GPIO_Pin_9);
    }

}

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
	CanRxMsg RxMessage;
	int8_t err;

	SystemInit();

  /*!< At this stage the microcontroller clock setting is already configured to
       168 MHz, this is done through SystemInit() function which is called from
       startup file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f4xx.c file
     */

  /*Initialize Leds */
  LED_Init();

  /*Configure the Interrupt*/
  InterruptCAN_Init();

  CAN_Initialisierung();

    xQueueFromCAN2 = xQueueCreate( 100, sizeof( RxMessage ) );
    xQueueToCAN1 = xQueueCreate( 100, sizeof( RxMessage ) );

    if(  xQueueFromCAN2 == 0 || xQueueToCAN1 == 0)
    {
    	while(1)
    	;
    }

  /* Create Tasks */
  err=xTaskCreate( vTask_CAN2_MessageReceived ,NULL,1048, NULL, REC_TASK_PRIO, NULL );
  if(err!=1)
	  while(1);
  err=xTaskCreate( vTask_Send_To_CAN1, NULL, 1048, NULL, SEND_TASK_PRIO, NULL);
  if(err!=1)
	  while(1);
  err=xTaskCreate( vTask_Reg, NULL, 130, NULL, REG_TASK_PRIO, NULL);
  if(err!=1)
	  while(1);

 /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
 /*Enable CAN Interrupt*/
  CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);

  /* Start scheduler */
  vTaskStartScheduler();

  /* We should never get here as control is now taken by the scheduler */
  for( ;; );
}



/**
  * @brief  Initializes the LEDs resources.
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
#ifdef USE_LCD
  /* Initialize the STM324xG-EVAL's LCD */
  STM324xG_LCD_Init();
#endif

  /* Initialize Discovery's LEDs */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

  /* Configure PD15 in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12|GPIO_Pin_13|  GPIO_Pin_14|  GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  /* Initialize PortE */
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);

  /* Configure PE in output pushpull mode */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_9 | GPIO_Pin_13;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOE, &GPIO_InitStructure);

}



/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
